/*
 *	Copyright (C) 2008, 2009, 2010. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file
@ingroup ExamplePrograms
@brief Example usage for the Designer User Interface

The Designer User Interface allows the user to dynamically modify the block and linking structure of a QVision application, adding new processing nodes to its structure, deleting them, or creating new data connections between them.

@section Usage Usage of the program.
Once QVision has been correctly installed in your system, compile using the <i>qmake</i> and <i>make</i> tools:
@code # qmake; make @endcode
Execute the application with the following line:
@code ./designer @endcode
For example:

See section @ref QVDesignerGUI for further info about the Designer Interface.

@author PARP Research Group. University of Murcia, Spain.
*/

#include <QVDesignerGUI>
#include <QVApplication>
#include <QVCannyEdgeDetector>
#include <QVMPlayerReaderBlock>
#include <QVImageCanvas>

#include <QVProcessingBlock>
#include <qvip.h>
#include <qvipp.h>
#include <qvmath.h>
#include <QV2DMap>

#ifndef DOXYGEN_IGNORE_THIS
class QVSplitC3ImageUChar: public QVProcessingBlock
	{
	public:
		QVSplitC3ImageUChar(QString name = QString()): QVProcessingBlock(name)
			{
			// Input properties
			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);

			// Output properties
			addProperty< QVImage<uChar,1> >("Channel 1 image", outputFlag);
			addProperty< QVImage<uChar,1> >("Channel 2 image", outputFlag);
			addProperty< QVImage<uChar,1> >("Channel 3 image", outputFlag);
			}

		void iterate()
			{
			// 0. Read input image.
			const QVImage<uChar,3> image = getPropertyValue< QVImage<uChar,3> >("Input image");
			QVImage<uChar,1> channel1, channel2, channel3;

			Copy(image, channel1, channel2, channel3);

			// 1. Export single-channel images
			setPropertyValue< QVImage<uChar,1> >("Channel 1 image", channel1);
			setPropertyValue< QVImage<uChar,1> >("Channel 2 image", channel2);
			setPropertyValue< QVImage<uChar,1> >("Channel 3 image", channel3);
			}
	};

class QVMergeC3ImageUChar: public QVProcessingBlock
	{
	public:
		QVMergeC3ImageUChar(QString name = QString()): QVProcessingBlock(name)
			{
			// Input properties
			addProperty< QVImage<uChar,1> >("Channel 1 input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,1> >("Channel 2 input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,1> >("Channel 3 input image", inputFlag|outputFlag);

			// Output properties
			addProperty< QVImage<uChar,3> >("Output image", outputFlag);
			}

		void iterate()
			{
			// 0. Read input images.
			const QVImage<uChar,1>	channel1 = getPropertyValue< QVImage<uChar,1> >("Channel 1 input image"),
						channel2 = getPropertyValue< QVImage<uChar,1> >("Channel 2 input image"),
						channel3 = getPropertyValue< QVImage<uChar,1> >("Channel 3 input image");

			QVImage<uChar,3> imageC3;
			
			Copy(channel1, channel2, channel3, imageC3);

			// 1. Export 3-channel image
			setPropertyValue< QVImage<uChar,3> >("Output image", imageC3);
			}
	};


class QVMyFilterHipass: public QVProcessingBlock
	{
	public:
		QVMyFilterHipass(QString name = QString()): QVProcessingBlock(name)
			{
			addProperty< QVImage<uChar,3> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,3> >("Output image", outputFlag);
			addProperty< IppiMaskSize >("mask", inputFlag, ippMskSize5x5);
			addProperty< QPoint >("destROIOffset", inputFlag);
			}

		void iterate()
			{
			// 0. Read input image.
			const QVImage<uChar, 3> qvimage_pSrc = getPropertyValue< QVImage<uChar, 3> > ("Input image");
			const IppiMaskSize mask = getPropertyValue< IppiMaskSize > ("mask");
			const QPoint destROIOffset = getPropertyValue< QPoint > ("destROIOffset");
		
			QVImage<uChar, 3> qvimage_pDst;
		
			FilterHipass(qvimage_pSrc, qvimage_pDst, mask, destROIOffset);
			setPropertyValue< QVImage<uChar, 3> >("Output image", qvimage_pDst);
			}
	};

class QVSFloatC1MaximalPoints: public QVProcessingBlock
	{
	public:
		QVSFloatC1MaximalPoints(QString name = QString()): QVProcessingBlock(name)
			{
			// Input properties
			addProperty< QVImage<sFloat> >("Input image", inputFlag|outputFlag);
			addProperty< double >("Threshold", inputFlag);
			addProperty< int >("Radius", inputFlag);

			// Output properties
			addProperty< QList<QPointF> >("max", outputFlag);
			}

		void iterate()
			{
			// 0. Read input image.
			const QVImage<sFloat> image = getPropertyValue< QVImage<sFloat> >("Input image");
			const double threshold = getPropertyValue< double >("Threshold");
			const int radius = getPropertyValue< int >("Radius");

			QMap<sFloat, QPointF> map = maximalPoints(image, threshold, radius);

			// 1. Export single-channel images
			setPropertyValue< QList<QPointF> >("max", map.values());
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv, "Example application for QVision.", true );
	
//	QVMPlayerReaderBlock videoReader("Video reader");

	QVDesignerGUI::registerUserType<QVSplitC3ImageUChar>("QVSplitC3ImageUChar");
	QVDesignerGUI::registerUserType<QVMergeC3ImageUChar>("QVMergeC3ImageUChar");
	QVDesignerGUI::registerUserType<QVMyFilterHipass>("QVMyFilterHipass");
	QVDesignerGUI::registerUserType<QVSFloatC1MaximalPoints>("QVSFloatC1MaximalPoints");

	QVSplitC3ImageUChar splitter("Splitter");
	QVMergeC3ImageUChar merger("Merger");
        QVMyFilterHipass hipass("Hipass");
        QVSFloatC1MaximalPoints max("Maximal");

//	videoReader.linkProperty(&canny, "Input image");

	QVDesignerGUI interface;

        /*QVMPlayerReaderBlock camera("Video");
	QVImageCanvas cannyDisplayer("Canny");

	camera.linkProperty(&canny, "Input image");
	canny.linkProperty("Input image", cannyDisplayer);
	canny.linkProperty("Output contours", cannyDisplayer);*/

	return app.exec();
	}

#endif
