/*
 *	Copyright (C) 2007, 2008, 2009, 2010. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PROJECTIVE_H
#define PROJECTIVE_H

//#include <QPair>
#include <qvmath.h>
#include <QVMatrix>
#include <QV3DPointF>

#ifdef QVIPP
#include <QVImage>
#include <qvipp.h>
#endif

/// @file
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////// HOMOGRAPHY CALIBRATION FUNCTION ///////////////////////////////////////////////

void homogenizePoints(const QList< QPointFMatching > &matchings,
	QVMatrix &premult, QVMatrix &postmult,
	QList< QPair<QPointF, QPointF> > &homogeneizedPairs);

/*
@brief Obtains an homography from two lists of corresponding points.
@ingroup qvprojectivegeometry

This function returns the homography that maps the points from a source position to a
destination position, according to a projective transormation.

@deprecated Use @ref ComputeProjectiveHomography instead
@param sourcePoints list of source points.
@param destinationPoints list of destination points.
*/
//QVMatrix ComputeHomography(const QList<QPointF> &sourcePoints, const QList<QPointF> &destPoints);

/*!
@brief Obtains a planar homography from a list of point correspondences.

This function obtains the homography matrix \f$ A \f$ which most closely maps the source points to their corresponding destination, in the input point matching list.
This homography matrix will represent a <a href="http://en.wikipedia.org/wiki/Projective_transformation">projective transformation</a>.

The function returns the matrix corresponding to the planar homography, from a list of four or more point correspondences between the location of those points at the source plane and their location in the destination plane.

Projective homography matrices can be used to obtain a frontal view of any planar figure appearing in an image. The following code and images shows this effect:

@image html rectification_homography.png

Image on the left is the original picture obtained with a pin-hole camera. Image on the right was obtained aplying a rectification homography to the pixels in the original image. This rectification homography is obtained using the function @ref ComputeProjectiveHomography with the four point correspondences between the points in the images featured with yellow dots:

@code
QList< QPair<QPointF, QPointF> > matchings;
matchings.append(QPair<QPointF, QPointF>(QPointF(-171,109),	QPointF(-100,+100)));
matchings.append(QPair<QPointF, QPointF>(QPointF(-120,31),	QPointF(-100,-100)));
matchings.append(QPair<QPointF, QPointF>(QPointF(117,53),	QPointF(+100,-100)));
matchings.append(QPair<QPointF, QPointF>(QPointF(11,115),	QPointF(+100,+100)));

const QVMatrix H = ComputeProjectiveHomography(matchings);
@endcode

The obtained matrix \f$ H \f$ holds the following equation:

\f$ H \propto \left(
\begin{array}{ccc}
	0.63868		& 0.77290	& -39.73059	\\
	-0.11082	& 1.94177	& -165.90578	\\
	-0.00034	& -0.00378	& 1		\\
\end{array} \right)\f$

Also for each point matching \f$ p \mapsto p' \f$ contained in the <i>matchings</i> list, the following equation holds.

\f$ p' \propto H p \f$

or

\f$ p' \times H p = 0 \f$

In C++ source code:
@code
QPair<QPointF, QPointF> matching;
foreach(matching, matchings)
	{
	const QVVector	p_0 = QVVector::homogeneousCoordinates(matching.first),
			p_1 = QVVector::homogeneousCoordinates(matching.second);
	std::cout << "p1 x H p0 =" << (p_1 ^ H*p_0) << std::endl;
	}
@endcode

The printed values are close to 0.

Any point \f$ p \f$ from the 2D plane can be mapped to another point in the plane \f$ q \f$ with an homography matrix \f$ H \f$ using the following C++ code:

@code
QPointF q = H * QVVector::homogeneousCoordinates(p);
@endcode

Or by using the @ref ApplyHomography functions.

@see ApplyHomography(const QVMatrix &, const QPointF &)
@see ApplyHomography(const QVMatrix &, const QList<QPointF> &)

@param matchings list of point matchings from the original location, to the destination location.
@ingroup qvprojectivegeometry
*/
QVMatrix ComputeProjectiveHomography(const QList< QPointFMatching > &matchings);

/*!
@brief Obtains an affine homography from a list of point correspondences.

This function obtains the homography matrix \f$ M \f$ which most closely maps the source points to their destination, in the input point matching list.
This homography matrix will represent an <a href="http://en.wikipedia.org/wiki/Affine_transformation">affine transformation</a>.

The function returns the matrix corresponding to the planar homography, from a list of three or more point correspondences between the location of those points at the source plane and their location in the destination plane.

Usage:
@code
QList< QPair<QPointF, QPointF> > matchings;
matchings.append(QPair<QPointF, QPointF>(QPointF(-171,109),	QPointF(-100,+100)));
matchings.append(QPair<QPointF, QPointF>(QPointF(-120,31),	QPointF(-100,-100)));
matchings.append(QPair<QPointF, QPointF>(QPointF(117,53),	QPointF(+100,-100)));

const QVMatrix M = ComputeAffineHomography(matchings);
@endcode

Any point \f$ p \f$ from the 2D plane can be mapped to another point in the plane \f$ q \f$ with an affine matrix \f$ M \f$ using the following C++ code:

@code
QPointF q = M * QVVector::homogeneousCoordinates(p);
@endcode

Or by using the @ref ApplyHomography functions.

@see ApplyHomography(const QVMatrix &, const QPointF &)
@see ApplyHomography(const QVMatrix &, const QList<QPointF> &)

@param matchings list of point matchings from the original location, to the destination location.
@ingroup qvprojectivegeometry
*/
QVMatrix ComputeAffineHomography(const QList< QPointFMatching > &matchings);

/*! @brief Obtain an euclidean mapping for a set of inter-image point matchings.

This function obtains an euclidean mapping between the source an destination locations of a set of point matchings. The mapping is returned as a \f$ 3 \times 3 \f$ matrix which can be multiplied to the source location of each mapping point in homogeneous coordinates, obtaining a location close to the destination location from the matching.

@todo write documentation for this function
@ingroup qvprojectivegeometry
*/
QVMatrix ComputeEuclideanHomography(const QList< QPointFMatching > &matchings);

/*
@brief Obtains the fundamental matrix between two images, using the 8 point algorithm.
@ingroup qvprojectivegeometry
This function performs point normalization to robustly obtain the F matrix.

@param matchings list of 8 point matchings
*/
//QVMatrix ComputeFundamentalMatrix(const QList< QPointFMatching > &matchings);

/*!
@brief Obtains the fundamental matrix between two images, using the 8 point algorithm.

This function performs point normalization to robustly obtain the F matrix.

@note This function is based on functionallity from the OpenCV library. Thus compatibility
with that library must be enabled to be available.
@param matchings list of 8 point matchings

@ingroup qvprojectivegeometry
*/
#ifdef QVOPENCV
QVMatrix cvFindFundamentalMat(const QList< QPointFMatching > &matchings);
#endif

/*!
@brief Maps a point using an homography

This function maps a point \f$ (x, y) \f$ in the 2D plane, using a planar homography. The homography is represented as a \f$ H_{3 \times 3} \f$ matrix. These matrices can be obtained using methods like @ref ComputeEuclideanHomography, @ref ComputeProjectiveHomography, or @ref ComputeAffineHomography from a set of point correspondences between the original 2D plane, and the mapped plane.

The output of the function is a point \f$ (x', y') \f$ which holds the following equation:

\f$ \left(\begin{array}{c} x' \\ y' \\ 1 \end{array}\right) \propto H \left(\begin{array}{c} x \\ y \\ 1 \end{array}\right) \f$

@see ComputeEuclideanHomography
@see ComputeAffineHomography
@see ComputeProjectiveHomography

@param homography The homography transformation matrix
@param point Point to apply the homography transformation
@ingroup qvprojectivegeometry
*/
#ifdef QVIPP
QPointF ApplyHomography(const QVMatrix &homography, const QPointF &point);
#endif

/*!
@brief Maps a set of points using an homography

This is an overloaded version of the @ref ApplyHomography(const QVMatrix &, const QPointF &) function provided by convenience. This funcion takes a list of points from the 2D plane, and an homography matrix as inputs. The output will be a list of points obtained by mapping the points from the input list, using the homography.

@see ApplyHomography(const QVMatrix &, const QPointF &)
@see ComputeEuclideanHomography
@see ComputeAffineHomography
@see ComputeProjectiveHomography

@param homography The homography transformation matrix
@param sourcePoints Points to apply the homography transformation
@ingroup qvprojectivegeometry
*/
#ifdef QVIPP
QList<QPointF> ApplyHomography(const QVMatrix &homography, const QList<QPointF> &sourcePoints);
#endif
/*!
@brief Performs an homography distortion on an image

The homography is represented as a \f$ H_{3 \times 3} \f$ matrix. These matrices can be obtained using methods like @ref ComputeEuclideanHomography, @ref ComputeProjectiveHomography, or @ref ComputeAffineHomography.

This function takes a single channel image, and an homography matrix as inputs. Each point  \f$ (x, y) \f$ in the input image is mapped to its location in the resulting image  \f$ (x', y') \f$  using the homography \f$ H \f$ as follows:

\f$ \left(\begin{array}{c} x' \\ y' \\ 1 \end{array}\right) \propto H \left(\begin{array}{c} x \\ y \\ 1 \end{array}\right) \f$

@see ComputeEuclideanHomography
@see ComputeAffineHomography
@see ComputeProjectiveHomography

@param homography The homography transformation matrix
@param image The input image to distort
@param interpolation Type of interpolation. Possible values for this parameter are:
	- IPPI_INTER_NN. Nearest neighbor interpolation
	- IPPI_INTER_LINEAR. Linear interpolation
	- IPPI_INTER_CUBIC. Cubic interpolation

@note This function is based on functionallity from the Intel IPP library. Thus compatibility
with that library must be enabled to be available.

@ingroup qvprojectivegeometry
*/
#ifdef QVIPP
QVImage<uChar, 1> ApplyHomography(const QVMatrix &homography, const QVImage<uChar, 1> &image, const int interpolation = IPPI_INTER_CUBIC);
#endif
/*!
@brief Performs an homography distortion on an image

The homography is represented as a \f$ H_{3 \times 3} \f$ matrix. These matrices can be obtained using methods like @ref ComputeEuclideanHomography, @ref ComputeProjectiveHomography, or @ref ComputeAffineHomography.

This function takes a three channel image, and an homography matrix as inputs. Each point \f$ (x, y) \f$ in the input image is mapped to its location in the resulting image  \f$ (x', y') \f$  using the homography \f$ H \f$ as follows:

\f$ \left(\begin{array}{c} x' \\ y' \\ 1 \end{array}\right) \propto H \left(\begin{array}{c} x \\ y \\ 1 \end{array}\right) \f$

@see ComputeEuclideanHomography
@see ComputeAffineHomography
@see ComputeProjectiveHomography

@param homography The homography transformation matrix
@param image The input image to distort
@param interpolation Type of interpolation. Possible values for this parameter are:
	- IPPI_INTER_NN. Nearest neighbor interpolation
	- IPPI_INTER_LINEAR. Linear interpolation
	- IPPI_INTER_CUBIC. Cubic interpolation

@note This function is based on functionallity from the Intel IPP library. Thus compatibility
with that library must be enabled to be available.
@ingroup qvprojectivegeometry
*/
#ifdef QVIPP
QVImage<uChar, 3> ApplyHomography(const QVMatrix &homography, const QVImage<uChar, 3> &image, const int interpolation = IPPI_INTER_CUBIC);
#endif

/*!
@brief Function to test if a 3x3 matrix corresponds to an homography.

Every matrix which holds a perspective deformation from one plane to another should validate some constrains. The most important is that its two first columns
should be perpendicular, and with a similar size, because for the matrix to correspond to an homography, they should be contained in a base of a rotated coordinate system.
Given the following homography matrix:

\f$ H = \left[ R_1 R_2 -CR^t \right] \f$

The error value returned by this function for it will be:

\f$ error = \left| \frac{ \|R_1\| - \|R_2\| }{ \|R_1\| + \|R_2\| } \right| + \left| \frac{ R_1 \cdot R_2}{ \|R_1\| \|R_2\| } \right| \f$

That error is a measure of the difference of sizes between the norm of the two column vectors of the homography, corresponding to the two first columns of the rotation matrix, and their dot product. When both values are close to zero, the matrix corresponds to an homography, and it won't otherwise.

A good method to prove that a matrix corresponds to an homography using this function, can be done testing the return value with a threshold of aproximately 0.3. If the return value of this function for a matrix is lower than this threshold, that matrix is likely to correspond to an homography, and is not likely to correspond otherwise.

@param homography a possible homography matrix.
@return a value close to 1 when the matrix doesn't corresponds to an homography, and close to 0 when it is close to be an homography.
@ingroup qvprojectivegeometry
*/
double HomographyTestError(const QVMatrix &homography);

/*!
@todo write documentation for this function

@ingroup qvprojectivegeometry
*/
void GetExtrinsicCameraMatrixFromHomography(const QVMatrix &K, const QVMatrix &H, QVMatrix &M4x4);

/*!
@brief Diagonal intrinsic camera matrix calibration


This function obtains a diagonal intrinsic camera matrix, consisting on the focal distance only. This matrix is such that

\f$ H = K \left[ R | t\right]	\f$

Where \f$ R \f$ is a rotation matrix. This function returns a direct approximation for the \f$ K \f$ matrix.

@todo rename to GetDirectCameraIntrinsicsFromPlanarHomography
@see GetIntrinsicCameraMatrixFromHomography
@ingroup qvprojectivegeometry
*/
void GetDirectIntrinsicCameraMatrixFromHomography(const QVMatrix &H, QVMatrix &K);

/*!
@brief Diagonal intrinsic camera matrix calibration

This function obtains a diagonal intrinsic camera matrix, consisting on the focal distance only. This matrix is such that

\f$ H = K \left[ R | t\right]	\f$

Where \f$ R \f$ is a rotation matrix. This matrix is obtained with a minimization process, so its result is more precise than that obtained with @ref GetDirectIntrinsicCameraMatrixFromHomography function.

@see GetDirectIntrinsicCameraMatrixFromHomography
@ingroup qvprojectivegeometry
*/
//void GetIntrinsicCameraMatrixFromHomography(const QVMatrix &H, QVMatrix &K,
//	double focal = 3, const double maxFocal = 50, const int maxIterations = 100, const double maxError = 0.00001);

/*!
@ingroup qvprojectivegeometry
@todo write documentation for this function
*/
void CalibrateCameraFromPlanarHomography(const QVMatrix &H, QVMatrix &K, QVMatrix &Rt);

/*!
@brief Obtains the intrinsic camera matrix from a planar homography

This functions obtains the intrinsic calibration matrix \f$ K \f$ corresponding to a simple pinhole camera model. The intrinsic camera matrix has only one free parameter, related to the focal distance of the camera:

\f$
K = \left(\begin{array}{ccc}
f	& 0	& 0	\\
0	& f	& 0	\\
0	& 0	& 1	\\
\end{array}\right)
\f$

This function should receive a planar homography corresponding to the mapping of a set of know template points, to an image frame captured with the camera containing a view of that template.

The following is an example of a full intrinsic camera calibration, knowing a set of point matchings between the template image and the input image read from the camera:

@code
[...]
QList< QPointFMatching > matchings;
matchings.append(QPointFMatching(QPointF(-171,109),	QPointF(-100,+100)));
matchings.append(QPointFMatching(QPointF(-120,31),	QPointF(-100,-100)));
matchings.append(QPointFMatching(QPointF(117,53),	QPointF(+100,-100)));
matchings.append(QPointFMatching(QPointF(11,115),	QPointF(+100,+100)));

QVMatrix H, K;
H = ComputeProjectiveHomography(matchings);
GetPinholeCameraIntrinsicsFromPlanarHomography(H, K);
[...]
@endcode

For further understanding of the planar homography calibration process, see @ref ComputeProjectiveHomography function documentation.

@param H			Input planar homography.
@param K			Matrix to store the intrinsic camera matrix
@param iterations		Cumber of iterations to perform camera calibration
@param maxGradientNorm		Minimal value of the gradient size (norm 2) to stop the minimization when reached.
@param step			Corresponds to parameter <i>step</i> for the <i>gsl_multimin_fdfminimizer_set</i> function.
@param tol			Corresponds to parameter <i>tol</i> for the <i>gsl_multimin_fdfminimizer_set</i> function.

@return a value close to 1 when the matrix doesn't corresponds to an homography, and close to 0 when it is close to be an homography.
@see ComputeProjectiveHomography
@ingroup qvprojectivegeometry
*/
void GetPinholeCameraIntrinsicsFromPlanarHomography(	const QVMatrix &H, QVMatrix &K, const int iterations = 100,
						const double maxGradientNorm = 1e-3, const double step = 0.01, const double tol = 1e-4);

/*!
@brief Obtains the canonical matrices corresponding to an essential matrix

See section 9.6.2 from <i>Multiple View Geometry in Computer Vision</i>.

@param E The input essential matrix.
@return A list containing the two possible canonocal camera matrices
@ingroup qvprojectivegeometry
*/
QList<QVMatrix> getCanonicalCameraMatricesFromEssentialMatrix(const QVMatrix &E);

/*!
@brief Obtains the essential matrix corresponding to a canonical camera matrix

This method estimates the essential matrix of a two view scenario, provided the second camera matrix \f$ P_2 \f$, considering the first camera matrix as the identity: \f$ P_1 = \left[ I | 0 \right] \f$
See section 9.6.1 from <i>Multiple View Geometry in Computer Vision</i>.

@param P The input canonical matrix.
@return The essential matrix corresponding to P.
@ingroup qvprojectivegeometry
*/
QVMatrix getEssentialMatrixFromCanonicalCameraMatrix(const QVMatrix &P);

/*!
@brief Obtains the canonical camera matrix corresponding to a set of 3D point to image point matchings.

The following formula models the relation between a set of points from the 3D world \f$ \{ X_i \}_{i=1..n} \f$ and their projections \f$ \{ x_i \}_{i=1..n} \f$ at an image:

\f$ x_i \propto P X_i \f$

This function uses a <a href="http://en.wikipedia.org/wiki/Direct_linear_transformation">DLT</a> to obtain the camera matrix \f$ P \f$ from a given set of 3D points and their corresponding image points.

@param points2d List containing the points from the image.
@param points3d List containing the 3D points.
@return The canonical camera matrix.
@ingroup qvprojectivegeometry
*/
QVMatrix getCameraMatrixFrom2D3DPointCorrespondences(const QList<QPointF> &points2d, const QList<QV3DPointF> &points3d);

/*! @brief Eliminates errors in the rotation component of a canonical camera matrix using a QR decomposition

Function @ref refineExtrinsicCameraMatrixWithPolarDecomposition(const QVMatrix &) obtains a valid canonical camera
matrix <i>P = [R|t]</i> from a given initial approximation <i>P* = [R*|t*]</i>, using the Polar decomposition.

This function obtains a computationally more efficient approximation of the <i>P</i> matrix using a QR decomposition.
The rotation matrix <i>R</i> of the resulting <i>P</i> is not the closest rotation matrix to the initial <i>R*</i>
regarding the Frobenius norm \f$ ||R-R*||_F \f$.

Despite of that, this function can be used when a faster and slightly less accurate version of the best <i>P</i> is wanted.

@ingroup qvprojectivegeometry
*/
QVMatrix refineExtrinsicCameraMatrixWithQRDecomposition(const QVMatrix &P);

/*! @brief Eliminates errors in the rotation component of a canonical camera matrix using a Polar decomposition

Given a matrix <i>P* = [R*|t*]</i> which approximates a canonical camera matrix, this function obtains the closest
valid canonical matrix \f$ P = [R|t]\f$. The rotation matrix of this valid canonical matrix must be an orthogonal matrix.
Thus it must fullfill:

	\f$ R^T R = I \f$

This function uses a correcting square 3x3 matrix <i>E<sup>-1</sup></i> that satisfies the following equation:

	\f$ E^-1 P* = P (1) \f$

This matrix <i>E<sup>-1</sup></i> also satisfies that the matrix <i>R</i> is the rotation matrix closest to <i>R*</i>
in a certain sense. Using the polar decomposition of <i>R*</i>, the function obtains the <i>E</i> which corrects it
to the closest valid rotation matrix <i>R</i> regarding the Frobenius norm \f$ ||R-R*||_F \f$.

The polar decomposition of the transpose of matrix <i>R*</i> is the following:

	\f$ R* = D^T U^T \f$

Where <i>U</i> is a rotation matrix and <i>D</i> is a positive-semidefinite Hermitian matrix. The matrix
<i>D<sup>T</sup></i> is used as the matrix <i>E</i>.

@see QVMatrix refineExtrinsicCameraMatrixWithQRDecomposition(const QVMatrix &)
@ingroup qvprojectivegeometry
*/
QVMatrix refineExtrinsicCameraMatrixWithPolarDecomposition(const QVMatrix &P);

/*!
@brief Recovers the location of a 3D point from its projection on several images and their corresponding canonical camera matrices.

Given the projection formula:

\f$ x \propto P X \f$

The location of the 3D point \f$ X \f$ can be triangulated with its projection on two or more images.

This function triangulates the location of a 3D point \f$ X \f$ from the locations of two or more image projections \f$ \{ x_i \}_{i=1..n} \f$ and the corresponding canonical camera matrices for these images \f$ \{ P_i \}_{i=1..n} \f$.

The method used is described at section 12.2 from <i>Multiple View Geometry in Computer Vision</i>.

@param points List containing the points from the image.
@param Plist List containing the canonical camera matrices.
@return The triangulated location for the point.
@ingroup qvprojectivegeometry
*/
QV3DPointF triangulate3DPointFromNViews(const QList<QPointF> &points, const QList<QVMatrix> &Plist);

/*!
@brief Recovers the location of a 3D point from its projection on two images and their corresponding canonical camera matrices.

Given the projection formula:

\f$ x \propto P X \f$

The location of the 3D point \f$ X \f$ can be triangulated with its projection on two images.

This function triangulates the location of a 3D point \f$ X \f$ from the locations of two image projections \f$ x_1 \f$, \f$ x_2 \f$ and the corresponding canonical camera matrices for these images \f$ P_1 \f$, \f$ P_2 \f$.

The method used is described at section 12.2 from <i>Multiple View Geometry in Computer Vision</i>.

@param point1 The projected location of the 3D point in the first image.
@param P1 The camera matrix for the first image.
@param point2 The projected location of the 3D point in the second image.
@param P2 The camera matrix for the second image.
@return The triangulated location for the point.
@ingroup qvprojectivegeometry
*/
QV3DPointF triangulate3DPointFrom2Views(const QPointF &point1, const QVMatrix &P1, const QPointF &point2, const QVMatrix &P2);

#endif
